package arithmetic

import combinatorParsing.{TokenizingParsers, BuildingParsers, Position}

object ArithmeticParser extends TokenizingParsers with BuildingParsers {
  delimiters.incl('(', ')', '+', '-', '*', '/',  '>', '<', '=')
  keywords.incl("if", "then", "else")

  type Output = Int

  /** <pre>
   *    Expr ::= Term { ('+' Term) | ('-' Term) }
   *           | 'if' Expr 'then' Expr 'else' Expr
   *  </pre>
   */
  def Expr: Parser = production(
      Term & rep(kw("+") & Term | kw("-") & Term))

  //to be completed

  /** <pre>
   *    Term ::= Factor { ('*' Factor) | ('/' Factor) }
   *  </pre>
   */
  def Term: Parser = production(Factor & rep(kw("*") & Factor | kw("/") & Factor))

  /** <pre>
   *    Factor ::= <number> | '(' Expr ')'
   *  </pre>
   */
  def Factor = production(numericLit
      | kw("(") & Expr & kw(")"))

  def build(xs: List[Token], pos: Position): Output = eval(xs)

  import implicitConversions._

  def eval(xs: List[Token]): Output = xs match {
    case t1 :: KW("+") :: t2 :: rest =>
      eval((t1 + t2) :: rest)
    case t1 :: KW("-") :: t2 :: rest =>
      eval((t1 - t2) :: rest)
    case t1 :: KW("*") :: t2 :: rest =>
      eval((t1 * t2) :: rest)
    case t1 :: KW("/") :: t2 :: rest =>
      eval((t1 / t2) :: rest)
    case NumericLit(n) :: Nil => Integer.parseInt(n)
    case KW("(") :: t :: KW(")") :: Nil => t
    //to be completed
    case NonTerminal(v) :: Nil =>
      v
  }

  def test(in: String): String = {
    run(Expr, in).toString
  }

  def main(args: Array[String]): Unit = {
    Console.println(test("1 + 2 - 3*2"))
    Console.println(test("(1 + 2) * 3"))
    Console.println(test("if (2 > 3) then 2 else 3"))
    Console.println(test("10 + (if (2) then 2 else 3)"))
    Console.println(test("10 + (if (2*3 = 3*2) then 2 else 3)"))
    Console.println(test("1 > 0"))
    Console.println(test("0 > 0 = 0"))
  }
}
